# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class System:
    @property
    def current_time(self):
        ret = self.command(
            'global.cgi?action=getCurrentTime'
        )
        return ret.content.decode('utf-8')

    @current_time.setter
    def current_time(self, date):
        """
        According with API:
            The time format is "Y-M-D H-m-S". It’s not be effected by Locales.
            TimeFormat in SetLocalesConfig

        Params:
            date = "Y-M-D H-m-S"
            Example: 2016-10-28 13:48:00

        Return: True
        """
        ret = self.command(
            'global.cgi?action=setCurrentTime&time={0}'.format(date)
        )

        if "ok" in ret.content.decode('utf-8').lower():
            return True

        return False

    def __get_config(self, config_name):
        ret = self.command(
            'configManager.cgi?action=getConfig&name={0}'.format(config_name)
        )
        return ret.content.decode('utf-8')

    @property
    def general_config(self):
        return self.__get_config('General')

    @property
    def version_http_api(self):
        ret = self.command(
            'IntervideoManager.cgi?action=getVersion&Name=CGI'
        )
        return ret.content.decode('utf-8')

    @property
    def software_information(self):
        ret = self.command(
            'magicBox.cgi?action=getSoftwareVersion'
        )
        return ret.content.decode('utf-8')

    @property
    def hardware_version(self):
        ret = self.command(
            'magicBox.cgi?action=getHardwareVersion'
        )
        return ret.content.decode('utf-8')

    @property
    def device_type(self):
        ret = self.command(
            'magicBox.cgi?action=getDeviceType'
        )
        return ret.content.decode('utf-8')

    @property
    def serial_number(self):
        ret = self.command(
            'magicBox.cgi?action=getSerialNo'
        )
        return ret.content.decode('utf-8')

    @property
    def machine_name(self):
        ret = self.command(
            'magicBox.cgi?action=getMachineName'
        )
        return ret.content.decode('utf-8')

    @property
    def system_information(self):
        ret = self.command(
            'magicBox.cgi?action=getSystemInfo'
        )
        return ret.content.decode('utf-8')

    @property
    def vendor_information(self):
        ret = self.command(
            'magicBox.cgi?action=getVendor'
        )
        return ret.content.decode('utf-8')

    @property
    def onvif_information(self):
        ret = self.command(
            'IntervideoManager.cgi?action=getVersion&Name=Onvif'
        )
        return ret.content.decode('utf-8')

    @property
    def config_backup(self, filename=None):
        ret = self.command(
            'Config.backup?action=All'
        )

        if filename is not None:
            with open(filename, "w+") as f:
                f.write(ret.content.decode('utf-8'))
            return

        return ret.content.decode('utf-8')

    @property
    def device_class(self):
        """
        During the development, device IP2M-841B didn't
        responde for this call, adding it anyway.
        """
        ret = self.command(
            'magicBox.cgi?action=getDeviceClass'
        )
        return ret.content.decode('utf-8')

    def shutdown(self):
        """
        From the testings, shutdown acts like "reboot now"
        """
        ret = self.command(
            'magicBox.cgi?action=shutdown'
        )
        return ret.content.decode('utf-8')

    def reboot(self, delay=None):
        cmd = 'magicBox.cgi?action=reboot'

        if delay is not None:
            cmd += "&delay={0}".format(delay)

        ret = self.command(cmd)
        return ret.content.decode('utf-8')



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et
import shutil


class Audio:

    @property
    def audio_input_channels_numbers(self):
        ret = self.command(
            'devAudioInput.cgi?action=getCollect'
        )
        return ret.content.decode('utf-8')

    @property
    def audio_output_channels_numbers(self):
        ret = self.command(
            'devAudioOutput.cgi?action=getCollect'
        )
        return ret.content.decode('utf-8')

    def audio_stream_capture(self, httptype=None,
                             channel=None, path_file=None):
        """
        Params:

            path_file - path to output file
            channel: - integer
            httptype - type string (singlepart or multipart)

                singlepart: HTTP content is a continuos flow of audio packets
                multipart: HTTP content type is multipart/x-mixed-replace, and
                           each audio packet ends with a boundary string

        """
        if httptype is None and channel is None:
            raise RuntimeError("Requires htttype and channel")

        ret = self.command(
            'audio.cgi?action=getAudio&httptype={0}&channel={1}'.format(
                httptype, channel))

        if path_file is not None:
            with open(path_file, 'wb') as out_file:
                shutil.copyfileobj(ret.raw, out_file)

        return ret.raw



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class Ptz:

    @property
    def ptz_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=Ptz'
        )
        return ret.content.decode('utf-8')

    @property
    def ptz_auto_movement(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=PtzAutoMovement'
        )
        return ret.content.decode('utf-8')

    def ptz_presets_list(self, channel=0):
        ret = self.command(
            'ptz.cgi?action=getPresets&channel={0}'.format(channel)
        )
        return ret.content.decode('utf-8')

    def ptz_status(self, channel=0):
        ret = self.command(
            'ptz.cgi?action=getStatus&channel={0}'.format(channel)
        )
        return ret.content.decode('utf-8')

    def ptz_tour_routines_list(self, channel=0):
        ret = self.command(
            'configManager.cgi?action=getTours&channel={0}'.format(channel)
        )
        return ret.content.decode('utf-8')

    def ptz_control_command(self, channel=0, action=None, code=None,
                            arg1=None, arg2=None, arg3=None):

        if action is None and code is None and arg1 is None and \
                arg2 is None and arg3 is None:
            raise RuntimeError("code, arg1, arg2, arg3 is required!")

        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code={2}&arg1={3}'
            '&arg2={4}&arg3={5}'.format(action, channel, code,
                                        arg1, arg2, arg3)
        )
        return ret.content.decode('utf-8')

    def zoom_in(self, action=None, channel=0):
        """
        Params:
            action              - start or stop
            channel             - channel number

        The magic of zoom in 1x, 2x etc. is the timer between the cmd
        'start' and cmd 'stop'. My suggestion for start/stop cmd is 0.5 sec
        """

        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=ZoomTele&arg1=0'
            '&arg2=0&arg3=0'.format(action, channel)
        )
        return ret.content.decode('utf-8')

    def zoom_out(self, action=None, channel=0):
        """
        Params:
            action              - start or stop
            channel             - channel number

        The magic of zoom out 1x, 2x etc. is the timer between the cmd
        'start' and cmd 'stop'. My suggestion for start/stop cmd is 0.5 sec
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=ZoomWide&arg1=0'
            '&arg2=0&arg3=0'.format(action, channel)
        )
        return ret.content.decode('utf-8')

    def move_left(self, action=None, channel=0, vertical_speed=1):
        """
        Params:
            action              - start or stop
            channel             - channel number
            vertical_speed      - range 1-8

        The magic of move left 1x, 2x etc. is the timer between the cmd
        'start' and cmd 'stop'. My suggestion for start/stop cmd is 0.5 sec
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=Left&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def move_right(self, action=None, channel=0, vertical_speed=1):
        """
        Params:
            action              - start or stop
            channel             - channel number
            vertical_speed      - range 1-8

        The magic of move right 1x, 2x etc. is the timer between the cmd
        'start' and cmd 'stop'. My suggestion for start/stop cmd is 0.5 sec
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=Right&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def move_up(self, action=None, channel=0, vertical_speed=1):
        """
        Params:
            action              - start or stop
            channel             - channel number
            vertical_speed      - range 1-8

        The magic of move up 1x, 2x etc. is the timer between the cmd
        'start' and cmd 'stop'. My suggestion for start/stop cmd is 0.2 sec
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=Up&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def move_down(self, action=None, channel=0, vertical_speed=1):
        """
        The magic of move down 1x, 2x etc. is the timer between the cmd
        'start' and cmd 'stop'. My suggestion for start/stop cmd is 0.2 sec
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=Down&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def focus_near(self, action=None, channel=0):
        """
        Params:
            action              - start or stop
            channel             - channel number
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=FocusNear&arg1=0'
            '&arg2=0&arg3=0'.format(action, channel)
        )
        return ret.content.decode('utf-8')

    def focus_far(self, action=None, channel=0):
        """
        Params:
            action              - start or stop
            channel             - channel number
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=FocusFar&arg1=0'
            '&arg2=0&arg3=0'.format(action, channel)
        )
        return ret.content.decode('utf-8')

    def iris_large(self, action=None, channel=0):
        """
        Aperture larger

        Params:
            action              - start or stop
            channel             - channel number
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=IrisLarge&arg1=0'
            '&arg2=0&arg3=0'.format(action, channel)
        )
        return ret.content.decode('utf-8')

    def iris_small(self, action=None, channel=0):
        """
        Aperture smaller

        Params:
            action              - start or stop
            channel             - channel number
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=IrisSmall&arg1=0'
            '&arg2=0&arg3=0'.format(action, channel)
        )
        return ret.content.decode('utf-8')

    def go_to_preset(self, action=None, channel=0, preset_point_number=1):
        """
        Params:
            action              - start or stop
            channel             - channel number
            preset_point_number - preset point number
        """
        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=GotoPreset&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, preset_point_number)
        )
        return ret.content.decode('utf-8')

    def move_left_up(self, action=None, channel=0,
                     vertical_speed=1, horizontal_speed=1):
        """
        Params:
            action           - start or stop
            channel          - channel number
            vertical_speed   - range is 1-8
            horizontal_speed - range is 1-8
        """

        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=LeftUp&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def move_left_down(self, action=None, channel=0,
                       vertical_speed=1, horizontal_speed=1):
        """
        Params:
            action           - start or stop
            channel          - channel number
            vertical_speed   - range is 1-8
            horizontal_speed - range is 1-8
        """

        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=LeftDown&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def move_right_up(self, action=None, channel=0,
                      vertical_speed=1, horizontal_speed=1):
        """
        Params:
            action           - start or stop
            channel          - channel number
            vertical_speed   - range is 1-8
            horizontal_speed - range is 1-8
        """

        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=RightUp&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def move_right_down(self, action=None, channel=0,
                        vertical_speed=1, horizontal_speed=1):
        """
        Params:
            action           - start or stop
            channel          - channel number
            vertical_speed   - range is 1-8
            horizontal_speed - range is 1-8
        """

        ret = self.command(
            'ptz.cgi?action={0}&channel={1}&code=RightDown&arg1=0'
            '&arg2={2}&arg3=0'.format(action, channel, vertical_speed)
        )
        return ret.content.decode('utf-8')

    def move_directly(self,
                      channel=1, startpoint_x=None, startpoint_y=None,
                      endpoint_x=None, endpoint_y=None):
        """

        Three-dimensional orientation. Move to the rectangle with screen
        coordinate [startX, startY], [endX, endY]

        Params:
            action           - start or stop
            channel          - channel index, start with 1
            startX, startY, endX and endY - range is 0-8192
        """

        if startpoint_x is None or startpoint_y is None or \
           endpoint_x is None or endpoint_y is None:
            raise RuntimeError("Required args, start_point_x, start_point_y"
                               "end_point_x and end_point_y not speficied")

        ret = self.command(
            'ptzBase.cgi?action=moveDirectly&channel={0}&startPoint[0]={1}'
            '&startPoint[1]={2}&endPoint[0]={3}&endPoint[1]={4}'.format(
             channel, startpoint_x, startpoint_y, endpoint_x, endpoint_y)
        )
        return ret.content.decode('utf-8')



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et
import requests

from .audio import Audio
from .system import System
from .network import Network
from .motion_detection import MotionDetection
from .snapshot import Snapshot
from .user_management import UserManagement
from .event import Event
from .record import Record
from .video import Video
from .log import Log
from .ptz import Ptz
from .special import Special


class Http(System, Network, MotionDetection, Snapshot,
           UserManagement, Event, Audio, Record, Video,
           Log, Ptz, Special):

    def __init__(self, host, port, user,
                 password, verbose=True, protocol='http'):
        self._host = host
        self._port = port
        self._user = user
        self._password = password
        self._verbose = verbose
        self._protocol = protocol
        self._token = requests.auth.HTTPBasicAuth(self._user, self._password)

    # Base methods
    def __base_url(self, param=""):
        return '%s://%s:%s/cgi-bin/%s' % (self._protocol, self._host,
                                          str(self._port), param)

    def command(self, cmd, timeout_cmd=3):
        """
        Args:
            cmd - command to execute via http
            timeout_cmd - timeout, default 3sec
        """
        try:
            url = self.__base_url(cmd)
            resp = requests.get(
                url,
                auth=self._token,
                stream=True,
                timeout=timeout_cmd,
            )
            resp.raise_for_status()
        except:
            raise

        return resp



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class MotionDetection:
    def __get_config(self, config_name):
        ret = self.command(
            'configManager.cgi?action=getConfig&name={0}'.format(config_name)
        )
        return ret.content.decode('utf-8')

    @property
    def motion_detection(self):
        return self.__get_config("MotionDetect")

    @motion_detection.setter
    def motion_detection(self, opt):
        if opt.lower() == "true" or opt.lower() == "false":
            ret = self.command(
                'configManager.cgi?action='
                'setConfig&MotionDetect[0].Enable={0}'.format(opt.lower())
            )
            if "ok" in ret.content.decode('utf-8').lower():
                return True

        return False



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et
import shutil


class Special:

    def realtime_stream(self, channel=1, typeno=0, path_file=None):
        """
        If the stream is redirect to a file, use mplayer tool to
        visualize the video record

        camera.realtime_stream(path_file="/home/user/Desktop/myvideo)
        $ mplayer /home/user/Desktop/myvideo
        """
        ret = self.command(
            'realmonitor.cgi?action=getStream&channel={0}&subtype={1}'.format(
                channel, typeno)
        )

        if path_file is not None:
            with open(path_file, 'wb') as out_file:
                shutil.copyfileobj(ret.raw, out_file)

        return ret.raw

    """
    11/05/2016

    Looks like the mjpeg support was removed from the mainstream in one of
    the firmware updates, so subtype=0 is not working.

    I have tested with the below firmware and still not working
    in the mainstream:
        Amcrest_IPC-AWXX_Eng_N_V2.420.AC00.15.R.20160908.bin

    Based on above, the cmd we send to camera should use: subtype=1 until
    mainstream is back.

    Simple translation:
        subtype=0 for mainstream
        subtype=1 for substream

    Tested on device: IP2M-841B

    Users complaining here too:
    https://amcrest.com/forum/technical-discussion-f3/lost-mjpeg-encode-for-main-stream-after-firmware-u-t1516.html

    """
    def mjpg_stream(self, channelno=None, typeno=None, path_file=None):
        """
        Params:
            channelno: integer, the video channel index which starts from 1,
                       default 1 if not specified.

            typeno: the stream type, default 0 if not specified. It can be
                    the following value:

                    0-Main Stream
                    1-Extra Stream 1 (Sub Stream)
                    2-Extra Stream 2 (Sub Stream)
        """
        if channelno is None and typeno is None:
            cmd = "mjpg/video.cgi?channel=0&subtype=1"
        else:
            cmd = "mjpg/video.cgi?channel={0}&subtype={1}".format(
                channelno, typeno)

        ret = self.command(cmd)

        if path_file is not None:
            with open(path_file, 'wb') as out_file:
                shutil.copyfileobj(ret.raw, out_file)

        return ret.raw



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class Record:

    @property
    def record_capability(self):
        ret = self.command(
            'recordManager.cgi?action=getCaps'
        )
        return ret.content.decode('utf-8')

    @property
    def record_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=Record'
        )
        return ret.content.decode('utf-8')

    @property
    def media_global_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=MediaGlobal'
        )
        return ret.content.decode('utf-8')



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class Video:

    @property
    def video_max_extra_stream(self):
        ret = self.command(
            'magicBox.cgi?action=getProductDefinition&name=MaxExtraStream'
        )
        return ret.content.decode('utf-8')

    @property
    def video_color_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=VideoColor'
        )
        return ret.content.decode('utf-8')

    @property
    def encode_capability(self):
        ret = self.command(
            'encode.cgi?action=getCaps'
        )
        return ret.content.decode('utf-8')

    def encode_config_capability(self, channel):
        ret = self.command(
            'encode.cgi?action=getConfigCaps&channel={0}'.format(channel)
        )
        return ret.content.decode('utf-8')

    @property
    def encode_media(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=Encode'
        )
        return ret.content.decode('utf-8')

    @property
    def encode_region_interested(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=VideoEncodeROI'
        )
        return ret.content.decode('utf-8')

    @property
    def video_channel_title(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=ChannelTitle'
        )
        return ret.content.decode('utf-8')

    @property
    def video_input_channels_device_supported(self):
        ret = self.command(
            'devVideoInput.cgi?action=getCollect'
        )
        return ret.content.decode('utf-8')

    @property
    def video_output_channels_device_supported(self):
        ret = self.command(
            'devVideoOutput.cgi?action=getCollect'
        )
        return ret.content.decode('utf-8')

    @property
    def video_max_remote_input_channels(self):
        ret = self.command(
            'magicBox.cgi?action=getProductDefinition&name='
            'MaxRemoteInputChannels'
        )
        return ret.content.decode('utf-8')

    @property
    def video_standard(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=VideoStandard'
        )
        return ret.content.decode('utf-8')

    @video_standard.setter
    def video_standard(self, std):
        ret = self.command(
            'configManager.cgi?action=setConfig&VideoStandard={0}'.format(std)
        )
        return ret.content.decode('utf-8')

    @property
    def video_widget_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=VideoWidget'
        )
        return ret.content.decode('utf-8')

    def video_input_capability(self, channel):
        ret = self.command(
            'devVideoInput.cgi?action=getCaps&channel={0}'.format(channel)
        )
        return ret.content.decode('utf-8')

    def video_coordinates_current_window(self, channel):
        ret = self.command(
            'devVideoInput.cgi?action='
            'getCurrentWindow&channel={0}'.format(channel)
        )
        return ret.content.decode('utf-8')

    @property
    def video_in_options(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=VideoInOptions'
        )
        return ret.content.decode('utf-8')

    @property
    def video_out_options(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=VideoOut'
        )
        return ret.content.decode('utf-8')



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class Event:

    def event_handler_config(self, handlername):
        ret = self.command(
            'configManager.cgi?action=getConfig&name={0}'.format(handlername)
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=Alarm'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_out_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=AlarmOut'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_input_channels(self):
        ret = self.command(
            'alarm.cgi?action=getInSlots'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_output_channels(self):
        ret = self.command(
            'alarm.cgi?action=getOutSlots'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_states_input_channels(self):
        ret = self.command(
            'alarm.cgi?action=getInState'
        )
        return ret.content.decode('utf-8')

    @property
    def alarm_states_output_channels(self):
        ret = self.command(
            'alarm.cgi?action=getOutState'
        )
        return ret.content.decode('utf-8')

    @property
    def video_blind_detect_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=BlindDetect'
        )
        return ret.content.decode('utf-8')

    @property
    def video_loss_detect_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=LossDetect'
        )
        return ret.content.decode('utf-8')

    @property
    def event_login_failure(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=LoginFailureAlarm'
        )
        return ret.content.decode('utf-8')

    @property
    def event_storage_not_exist(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=StorageNotExist'
        )
        return ret.content.decode('utf-8')

    @property
    def event_storage_access_failure(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=StorageFailure'
        )
        return ret.content.decode('utf-8')

    @property
    def event_storage_low_space(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=StorageLowSpace'
        )
        return ret.content.decode('utf-8')

    @property
    def event_net_abort(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=NetAbort'
        )
        return ret.content.decode('utf-8')

    @property
    def event_ip_conflict(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=IPConflict'
        )
        return ret.content.decode('utf-8')

    def event_channels_happened(self, eventcode):
        """
        Params:

        VideoMotion: motion detection event
        VideoLoss: video loss detection event
        VideoBlind: video blind detection event
        AlarmLocal: alarm detection event
        StorageNotExist: storage not exist event
        StorageFailure: storage failure event
        StorageLowSpace: storage low space event
        AlarmOutput: alarm output event
        """
        ret = self.command(
            'eventManager.cgi?action=getEventIndexes&code={0}'.format(
                eventcode)
        )
        return ret.content.decode('utf-8')

    @property
    def event_management(self):
        ret = self.command(
            'eventManager.cgi?action=getCaps'
        )
        return ret.content.decode('utf-8')



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et


class UserManagement:

    def info_user(self, username):
        ret = self.command(
            'userManager.cgi?action=getUserInfo&name={0}'.format(username)
        )
        return ret.content.decode('utf-8')

    @property
    def info_all_users(self):
        ret = self.command(
            'userManager.cgi?action=getUserInfoAll'
        )
        return ret.content.decode('utf-8')

    @property
    def info_all_active_users(self):
        ret = self.command(
            'userManager.cgi?action=getActiveUserInfoAll'
        )
        return ret.content.decode('utf-8')

    def info_group(self, group):
        ret = self.command(
            'userManager.cgi?action=getGroupInfo&name={0}'.format(group)
        )
        return ret.content.decode('utf-8')

    @property
    def info_all_groups(self):
        ret = self.command(
            'userManager.cgi?action=getGroupInfoAll'
        )
        return ret.content.decode('utf-8')

    def delete_user(self, username):
        ret = self.command(
            'userManager.cgi?action=deleteUser&name={0}'.format(username)
        )
        return ret.content.decode('utf-8')

    def add_user(self, username, password, group, sharable=True,
                 reserved=False, memo=None):
        """
        Params:
            username - username for user
            password - password for user
            group - string the range is "admin" and "user". In different group,
                    the user has different authorities.

            sharable - bool, true means allow multi-point login

            reserved - bool, true means this user can't be deleted

            memo - memo to user
        """

        cmd = "userManager.cgi?action=addUser&user.Name={0}" \
              "&user.Password={1}&user.Group={2}&user.Sharable={3}" \
              "&user.Reserved={4}".format(
                username, password, group.lower(), sharable.lower(),
                reserved.lower())

        if memo:
            cmd += "&user.Memo=%s" % memo

        ret = self.command(cmd)
        return ret.content.decode('utf-8')

    def modify_password(self, username, newpwd, oldpwd):
        """
        Params:
            username - user name
            newpwd - new password
            oldpwd - old password
        """
        ret = self.command(
            'userManager.cgi?action=modifyPassword&name={0}&pwd={1}'
            '&pwdOld={2}'.format(username, newpwd, oldpwd)
        )
        return ret.content.decode('utf-8')

    def modify_user(self, username, attribute, value):
        """
        Params:
            username - username for user
            attribute - the attribute name that will change:
                        group, sharable, reserved, memo

            value - the new value for attribute
        """

        cmd = "userManager.cgi?action=modifyUser&name={0}".format(
            username)

        if attribute.lower() == "group":
            cmd += "&user.Group=%s" % value.lower()

        elif attribute.lower() == "sharable":
            cmd += "&user.Sharable=%s" % value.lower()

        elif attribute.lower() == "reserved":
            cmd += "&user.Reserved=%s" % value.lower()

        elif attribute == "memo":
            cmd += "&user.Memo=%s" % value.lower()

        ret = self.command(cmd)
        return ret.content.decode('utf-8')



# -*- coding: utf-8 -*-
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# vim:sw=4:ts=4:et
from contextlib import closing
import socket
import threading


class Network:

    amcrest_ips = []
    __RTSP_PORT = 554
    __PWGPSI_PORT = 3800

    def __raw_scan(self, ip, timeout=None):
        # If devices not found, try increasing timeout
        socket.setdefaulttimeout(0.2)

        if timeout is not None:
            socket.setdefaulttimeout(timeout)

        with closing(socket.socket()) as sk:
            try:
                sk.connect((ip, self.__RTSP_PORT))
                sk.connect((ip, self.__PWGPSI_PORT))
                self.amcrest_ips.append(ip)
            except:
                pass

    def scan_devices(self, subnet, timeout=None):
        """
        Scan cameras in a range of ips

        Params:
        subnet - subnet, i.e: 192.168.1.0/24
                 if mask not used, assuming mask 24

        timeout_sec - timeout in sec

        Returns:
        """

        # Maximum range from mask
        # Format is mask: max_range
        max_range = {
            16: 256,
            24: 256,
            25: 128,
            27: 32,
            28: 16,
            29: 8,
            30: 4,
            31: 2
        }

        # If user didn't provide mask, use /24
        if "/" not in subnet:
            mask = int(24)
            network = subnet
        else:
            network, mask = subnet.split("/")
            mask = int(mask)

        if mask not in max_range:
            raise RuntimeError("Cannot determine the subnet mask!")

        # Default logic is remove everything from last "." to the end
        # This logic change in case mask is 16
        network = network.rpartition(".")[0]

        if mask == 16:
            # For mask 16, we must cut the last two
            # entries with .
            for i in range(0, 1):
                network = network.rpartition(".")[0]

        # Trigger the scan
        # For clear coding, let's keep the logic in if/else (mask16)
        # instead of only one if
        if mask == 16:
            for seq1 in range(0, max_range[mask]):
                for seq2 in range(0, max_range[mask]):
                    ip = "{0}.{1}.{2}".format(network, seq1, seq2)
                    t = threading.Thread(
                        target=self.__raw_scan, args=(ip, timeout)
                    )
                    t.start()
        else:
            for seq1 in range(0, max_range[mask]):
                ip = "{0}.{1}".format(network, seq1)
                t = threading.Thread(
                    target=self.__raw_scan, args=(ip, timeout)
                )
                t.start()

        return self.amcrest_ips

    @property
    def wlan_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=WLan'
        )
        return ret.content.decode('utf-8')

    @property
    def telnet_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=Telnet'
        )
        return ret.content.decode('utf-8')

    @telnet_config.setter
    def telnet_config(self, status):
        """
        status:
            false - Telnet is disabled
            true  - Telnet is enabled
        """
        ret = self.command(
            'configManager.cgi?action=setConfig&Telnet.Enable={0}'.format(
                status)
        )
        return ret.content.decode('utf-8')

    @property
    def network_config(self):
        ret = self.command(
            'configManager.cgi?action=getConfig&name=Network'
        )
        return ret.content.decode('utf-8')

    @property
    def network_interfaces(self):
        ret = self.command(
            'netApp.cgi?action=getInterfaces'
        )
        return ret.content.decode('utf-8')
